
import unittest

# TODO [template]: code this stub class.
class GetDiffPathsTestCase(unittest.TestCase):

    # TODO [template]: add a test case for excluding an item in the project
    # directory.
    def test(self):
        self.assertEquals(1, 1)


if __name__ == '__main__':
    unittest.main()
