# encoding: utf-8
# Project auto-generated at: ...
# Copyright (C) 2012 Mustachioed Maven.


# TODO: replace this placeholder docstring and function body with the
# real implementation.
def run_pizza(values):
    """
    Return the number of values.

    This function executes the Pizza package's main command-line functionality.

    """
    return len(values)
