"""
Contains code specific to using this project from the command-line.

In particular, this package contains the console_script entries for the
entry_points argument to setup.py's setup() function.
"""
