# encoding: utf-8
# Project auto-generated at: ...
# Copyright (C) 2012 Mustachioed Maven.

"""
A convenience script for running the main console script while developing.

"""

from pizza.scripts.pizza import main as main_module


def main():
    # The body of this function should be the same as the contents of the
    # __main__.py file in the subpackage containing main_module.
    main_module.main(from_source=True)

if __name__ == "__main__":
    main()
